use Test::More;
eval "use Test::Pod 1.00";
plan skip_all => "Test::Pod 1.00 required for testing POD" if $@;
plan skip_all => "set TEST_POD or TEST_ALL to enable this test"
    unless $ENV{TEST_POD} or $ENV{TEST_ALL};
all_pod_files_ok();
